#include <TrueRandom.h>

#include <Arduino.h>
#include <TM1637Display.h>
#include <DS3231.h>
#include <Adafruit_NeoPixel.h>
#include <EEPROM.h>


extern void setup_rtc(void);
extern void do_rtc(void);
extern void checkBeepNow(void);
extern void checkButtons(void);
void convertTime2Data(byte h, byte m);
void blinkColumn(void);
void clear(void);
extern void setup_ledstrip(void);
extern void rainbowCycle(void);
extern void executeAlarm(void);
extern uint32_t Wheel(byte WheelPos);
extern void writeToEeprom(unsigned int addr, byte value);
extern byte readFromEeprom(unsigned int addr, byte defaultValue);
void testEeprom();
void readInitValuesFromEeprom();


#define CLK 2
#define DIO 3
#define B1_PIN   4	//Button1 input
#define B2_PIN   5	//Button2 input
#define B3_PIN   6	//Button3 input
#define PIN_ALRMLED 7

#define BEEP 1
#define LED1 2
#define LED2 3
#define LED3 4

#define ALOF 1
#define ALON 2

#define CONFIG_IDLE 0
#define CONFIG_TIME_MINUTE 1
#define CONFIG_TIME_HOUR 2
#define CONFIG_ALRM_MINUTE 3
#define CONFIG_ALRM_HOUR 4
#define CONFIG_ALRM_ONOFF 5
#define CONFIG_ALRM_SHOW 6
#define CONFIG_TEMP_SHOW 7
#define CONFIG_BRIGHTNESS 8
#define CONFIG_ALRM_BEEP 9
#define CONFIG_ALRM_TYPE 10

extern byte s,m,h,dd,mm,yy,T; 
extern byte ADay, AHour, AMinute, ASecond, ABits;
TM1637Display display(CLK, DIO);
uint8_t data[] = { 0x00, 0x00, 0x00, 0x00 };
bool beginOfHalfSecondToggle = true;
bool blinkHourDigits = false;
bool blinkMinuteDigits = false;
bool blinkColumnDots = true;
byte B1cnt=0;
byte B2cnt=0;
byte B3cnt=0;
int configState = CONFIG_IDLE;
extern DS3231 Clock;
byte brightness; //A lire de l'eeprom
byte alarmType;  //A lire de l'eeprom
extern bool doAlarm;
extern Adafruit_NeoPixel strip;
byte cb=255, cr=0, cg=0;
byte cb0, cr0, cg0;
uint16_t wheelcycleIdx = 0;
extern bool forceAlarmManually;
bool forceShutdownDisplayAndAlarmLed = false;
byte configAlrmShowCounter = 0;
extern byte LED2cnt;
extern byte LED3bright;

void clear(void){
  data[0] = 0x00; data[1] = 0x00; data[2] = 0x00; data[3] = 0x00;
  display.setSegments(data);
}

void convertTime2Data(byte h, byte m){
  if (h<10){
    data[0] = 0x00;
    data[1] = display.encodeDigit(h);
  }
  if (h>=10 && h<20){
    data[0] = display.encodeDigit(1);
    data[1] = display.encodeDigit(h-10);
  }
  //data[1] |= SEG_X;
  if (h>=20){
    data[0] = display.encodeDigit(2);
    data[1] = display.encodeDigit(h-20);
  }
  if (m<10){
    data[2] = display.encodeDigit(0);
    data[3] = display.encodeDigit(m);
  }
  if (m>=10 && m<20){
    data[2] = display.encodeDigit(1);
    data[3] = display.encodeDigit(m-10);
  }
  if (m>=20 && m<30){
    data[2] = display.encodeDigit(2);
    data[3] = display.encodeDigit(m-20);
  }
  if (m>=30 && m<40){
    data[2] = display.encodeDigit(3);
    data[3] = display.encodeDigit(m-30);
  }
  if (m>=40 && m<50){
    data[2] = display.encodeDigit(4);
    data[3] = display.encodeDigit(m-40);
  }
  if (m>=50){
    data[2] = display.encodeDigit(5);
    data[3] = display.encodeDigit(m-50);
  }
}

void blinkColumnOrDigits(void){
  if (beginOfHalfSecondToggle){
    if (blinkHourDigits){
      data[0] = 0x00;
      data[1] = 0x00;
    }
    if (blinkMinuteDigits){
      data[2] = 0x00;
      data[3] = 0x00;
    }
    beginOfHalfSecondToggle = false;
  } else {
    if (blinkColumnDots){
      data[1] ^= SEG_X;
    }
    beginOfHalfSecondToggle = true;
  }
}

void testEeprom(){
  writeToEeprom(0, BRIGHT_1);
  writeToEeprom(1, LED2);
  writeToEeprom(2, ALON);
  writeToEeprom(3, 12);
  writeToEeprom(4, 34);
  
  brightness = readFromEeprom(0, BRIGHT_1);
  alarmType =  readFromEeprom(1, LED2);
  if (readFromEeprom(2, ALOF) == ALOF) Clock.turnOffAlarm(1); else Clock.turnOnAlarm(1);
  AHour = readFromEeprom(3, 6);
  AMinute = readFromEeprom(4, 30);
}

void readInitValuesFromEeprom(){
  brightness = readFromEeprom(0, BRIGHT_1);
  alarmType =  readFromEeprom(1, LED2);
  if (readFromEeprom(2, ALOF) == ALOF) Clock.turnOffAlarm(1); else Clock.turnOnAlarm(1);
  AHour = readFromEeprom(3, 6);
  AMinute = readFromEeprom(4, 30);
}

void setup()
{
  setup_rtc();
  setup_ledstrip();
//  testEeprom(); //setting some initial values into eeprom and reading it back for testing purpose.
  readInitValuesFromEeprom();
  setup_alarm();
}


void do_state_machine(){
  
  if (!digitalRead(B1_PIN)) B1cnt+=1;
  if (!digitalRead(B2_PIN)) B2cnt+=1;
  if (!digitalRead(B3_PIN)) B3cnt+=1;
/*
  Serial.print(" configState=");
  Serial.print(configState);
  Serial.print(" B1cnt=");Serial.print(B1cnt);
  Serial.print(" B2cnt=");Serial.print(B2cnt);
  Serial.print(" B3cnt=");Serial.print(B3cnt);
  Serial.print("\n");
*/
  switch (configState){
  case CONFIG_IDLE: /////////////////////////////////
    do_rtc();
    convertTime2Data(h,m);
    checkBeepNow();

   if (B2cnt > 10){//----------------Push long on B2 to test LED3 Alarm mode-----------------
      blinkMinuteDigits = false;
      blinkHourDigits = false;
      blinkColumnDots = true;
      LED3bright = B2cnt;
      if (digitalRead(B2_PIN)) B2cnt=0;
      if (B2cnt == 0){
        forceAlarmManually = true;
        alarmType = LED3;
        configState = CONFIG_IDLE;
      }
    }
    if (B2cnt>0 && B2cnt < 10){//----------push short on B2 to deactivate display-----------
      if (digitalRead(B2_PIN)) B2cnt=0;
      if (B2cnt == 0){
        forceShutdownDisplayAndAlarmLed = true;
        configState = CONFIG_IDLE;
        blinkMinuteDigits = false;
        blinkHourDigits = false;
        blinkColumnDots = true;
      }
    }
    if (B3cnt > 10){//----------------Push long on B3 to test LED2 Alarm mode----------------
      blinkMinuteDigits = false;
      blinkHourDigits = false;
      blinkColumnDots = true;
      LED2cnt = B3cnt;
      if (digitalRead(B3_PIN)) B3cnt=0;
      if (B3cnt == 0){
        forceAlarmManually = true;
        alarmType = LED2;
        configState = CONFIG_IDLE;
      }
    }
    if (B3cnt>0 && B3cnt < 10){//----------push short on B3 to activate display-----------
      if (digitalRead(B3_PIN)) B3cnt=0;
      if (B3cnt == 0){
        forceShutdownDisplayAndAlarmLed = false;
        configState = CONFIG_IDLE;
        blinkMinuteDigits = false;
        blinkHourDigits = false;
        blinkColumnDots = true;
      }
    }
    if (B1cnt > 10){//---------------Push long on B1 to go to config mode-----------------
      blinkMinuteDigits = true;
      blinkHourDigits = false;
      blinkColumnDots = true;
      if (digitalRead(B1_PIN)) B1cnt=0;
      if (B1cnt == 0){
        configState = CONFIG_TIME_MINUTE;
      }
    }
    if (B1cnt>0 && B1cnt < 10){//----------push short on B1 to display current Alarm time-----------
      if (digitalRead(B1_PIN)) B1cnt=0;
      if (B1cnt == 0){
        configState = CONFIG_ALRM_SHOW;
        blinkMinuteDigits = false;
        blinkHourDigits = false;
        blinkColumnDots = false;
      }
    }
    if (doAlarm){//----------------------------------
      configState = CONFIG_ALRM_BEEP;
      blinkMinuteDigits = false;
      blinkHourDigits = false;
      blinkColumnDots = true;
    }
    break;
  
  case CONFIG_ALRM_BEEP://///////////////////////////
    do_rtc();
    convertTime2Data(h,m);
    checkBeepNow();

    if (!doAlarm){ //-----------------------------------------------------
        configState = CONFIG_IDLE;
        blinkMinuteDigits = false;
        blinkHourDigits = false;
        blinkColumnDots = true;
    }
    if (B1cnt>0 || B2cnt>0 || (B3cnt>0 && B3cnt < 10)){ //--------------Stop alarm with any key------------------

      if (digitalRead(B1_PIN)) B1cnt=0;
      if (digitalRead(B2_PIN)) B2cnt=0;
      if (digitalRead(B3_PIN)) B3cnt=0;
      
      if (B3cnt==0){
      }
                
      if (B1cnt == 0 || B2cnt == 0 || B3cnt == 0){
        configState = CONFIG_IDLE;
        blinkMinuteDigits = false;
        blinkHourDigits = false;
        blinkColumnDots = true;
        doAlarm = false;
        forceAlarmManually = false;
        
        for(unsigned int i=0;i<strip.numPixels();i++){
          strip.setPixelColor(i, strip.Color(0,0,0)); 
        }
        strip.show();
      }
      
    }
    break;
  
  case CONFIG_ALRM_SHOW://///////////////////////////
    convertTime2Data(AHour,AMinute);
    data[1] |= SEG_X;
    
    configAlrmShowCounter += 1;
    
    if (configAlrmShowCounter == 50){
        configAlrmShowCounter = 0;
        configState = CONFIG_IDLE;
        blinkMinuteDigits = false;
        blinkHourDigits = false;
        blinkColumnDots = true;
    }
    
    if (B1cnt>0){
      if (digitalRead(B1_PIN)) B1cnt=0;
      if (B1cnt == 0){
        configState = CONFIG_TEMP_SHOW;
        blinkMinuteDigits = false;
        blinkHourDigits = false;
        blinkColumnDots = false;
      }
    }
    break;

  case CONFIG_TEMP_SHOW://///////////////////////////
    do_rtc();
    convertTime2Data(T,0);
    data[2] = SEG_A | SEG_F | SEG_G | SEG_B; //° 
    data[3] = SEG_A | SEG_F | SEG_E | SEG_D; //C
        
    if (B1cnt>0){
      if (digitalRead(B1_PIN)) B1cnt=0;
      if (B1cnt == 0){
        configState = CONFIG_IDLE;
        blinkMinuteDigits = false;
        blinkHourDigits = false;
        blinkColumnDots = true;
      }
    }
    break;
    
  case CONFIG_TIME_MINUTE:///////////////////////////
    convertTime2Data(h,m);

    if (B2cnt>0){
      if (digitalRead(B2_PIN)) B2cnt=0;
      if (B2cnt == 0) {
        m-=1; if (m==255) m=59;
        blinkMinuteDigits = false;
        convertTime2Data(h,m);
        Clock.setHour(h);Clock.setMinute(m);Clock.setSecond(00);
      }
    }
    if (B3cnt>0){
      if (digitalRead(B3_PIN)) B3cnt=0;
      if (B3cnt == 0) {
        m+=1; if (m==60) m=0;
        blinkMinuteDigits = false;
        convertTime2Data(h,m);
        Clock.setHour(h);Clock.setMinute(m);Clock.setSecond(00);
      }
    }    
    
    if (B1cnt>0){//--------------------------------
      if (digitalRead(B1_PIN)) B1cnt=0;
      if (B1cnt == 0){
        configState = CONFIG_TIME_HOUR;
        blinkMinuteDigits = false;
        blinkHourDigits = true;
        blinkColumnDots = true;
      }
    }
    break;
    
  case CONFIG_TIME_HOUR://///////////////////////////
    convertTime2Data(h,m);

    if (B2cnt>0){
      if (digitalRead(B2_PIN)) B2cnt=0;
      if (B2cnt == 0) {
        h-=1; if (h==255) h=23;
        blinkHourDigits = false;
        convertTime2Data(h,m);
        Clock.setHour(h);Clock.setMinute(m);Clock.setSecond(00);
      }
    }
    if (B3cnt>0){
      if (digitalRead(B3_PIN)) B3cnt=0;
      if (B3cnt == 0) {
        h+=1; if (h==24) h=0;
        blinkHourDigits = false;
        convertTime2Data(h,m);
        Clock.setHour(h);Clock.setMinute(m);Clock.setSecond(00);
      }
    }
    
    if (B1cnt>0){ //--------------------------------
      if (digitalRead(B1_PIN)) B1cnt=0;
      if (B1cnt == 0){
        configState = CONFIG_ALRM_MINUTE;
        blinkMinuteDigits = true;
        blinkHourDigits = false;
        blinkColumnDots = false; 
      }
    }
    break;

  case CONFIG_ALRM_MINUTE:///////////////////////////
    convertTime2Data(AHour,AMinute);

    if (B2cnt>0){
      if (digitalRead(B2_PIN)) B2cnt=0;
      if (B2cnt == 0) {
        AMinute-=1; if (AMinute==255) AMinute=59;
        blinkMinuteDigits = false;
        convertTime2Data(AHour,AMinute);
        Clock.setA1Time(0, AHour, AMinute, 00, 4, false, false, false);
      }
    }
    if (B3cnt>0){
      if (digitalRead(B3_PIN)) B3cnt=0;
      if (B3cnt == 0) {
        AMinute+=1; if (AMinute>=60) AMinute=0;
        blinkMinuteDigits = false;
        convertTime2Data(AHour,AMinute);
        Clock.setA1Time(0, AHour, AMinute, 00, 4, false, false, false);
      }
    }
    
    if (B1cnt>0){ //--------------------------------
      if (digitalRead(B1_PIN)) B1cnt=0;
      if (B1cnt == 0){
        configState = CONFIG_ALRM_HOUR;
        blinkMinuteDigits = false;
        blinkHourDigits = true;
        blinkColumnDots = false;
      }
    }
    break;
    
  case CONFIG_ALRM_HOUR://///////////////////////////
    convertTime2Data(AHour,AMinute);

    if (B2cnt>0){
      if (digitalRead(B2_PIN)) B2cnt=0;
      if (B2cnt == 0) {
        AHour-=1; if (AHour==255) AHour=23;
        blinkHourDigits = false;
        convertTime2Data(AHour,AMinute);
        Clock.setA1Time(0, AHour, AMinute, 00, 4, false, false, false);
        writeToEeprom(3, AHour);
        writeToEeprom(4, AMinute);
      }
    }
    if (B3cnt>0){
      if (digitalRead(B3_PIN)) B3cnt=0;
      if (B3cnt == 0) {
        AHour+=1; if (AHour==24) AHour=0;
        blinkHourDigits = false;
        convertTime2Data(AHour,AMinute);
        Clock.setA1Time(0, AHour, AMinute, 00, 4, false, false, false);
        writeToEeprom(3, AHour);
        writeToEeprom(4, AMinute);
      }
    }

    if (B1cnt>0){ //--------------------------------
      if (digitalRead(B1_PIN)) B1cnt=0;
      if (B1cnt == 0){
        configState = CONFIG_ALRM_ONOFF;
        blinkMinuteDigits = false;
        blinkHourDigits = false;
        blinkColumnDots = false;
      }
    }
    break;
    
  case CONFIG_ALRM_ONOFF:////////////////////////////
//      A
//     ---
//  F |   | B
//     -G-
//  E |   | C
//     ---
//      D

    if (Clock.checkAlarmEnabled(1)) {
      data[0] = SEG_E | SEG_F | SEG_A | SEG_G | SEG_B | SEG_C; //A
      data[1] = SEG_F | SEG_E | SEG_D;                         //L
      data[2] = SEG_E | SEG_G | SEG_C | SEG_D;                 //o
      data[3] = SEG_E | SEG_G | SEG_C;                         //n
    } else {
      data[0] = SEG_E | SEG_F | SEG_A | SEG_G | SEG_B | SEG_C; //A
      data[1] = SEG_F | SEG_E | SEG_D;                         //L
      data[2] = SEG_E | SEG_G | SEG_C | SEG_D;                 //o
      data[3] = SEG_E | SEG_F | SEG_A | SEG_G;                 //F
    }

    if (B2cnt>0){
      if (digitalRead(B2_PIN)) B2cnt=0;
      if (B2cnt == 0) {
        Clock.turnOffAlarm(1);
        writeToEeprom(2, ALOF);
      }
    }
    if (B3cnt>0){
      if (digitalRead(B3_PIN)) B3cnt=0;
      if (B3cnt == 0) {
        Clock.turnOnAlarm(1);
        writeToEeprom(2, ALON);
      }
    }
    
    if (B1cnt>0){ //--------------------------------
      if (digitalRead(B1_PIN)) B1cnt=0;
      if (B1cnt == 0){
        configState = CONFIG_BRIGHTNESS;
        blinkMinuteDigits = false;
        blinkHourDigits = false;
        blinkColumnDots = false;
      }
    }
    break;
    
  case CONFIG_BRIGHTNESS:///////////////////////////
//      A
//     ---
//  F |   | B
//     -G-
//  E |   | C
//     ---
//      D

    data[0] = SEG_F | SEG_E | SEG_D | SEG_C | SEG_G; //b
    data[1] = SEG_E | SEG_G;                         //r
    data[2] = SEG_G | SEG_F | SEG_A | SEG_B | SEG_C; //9
    data[3] = SEG_F | SEG_E | SEG_G | SEG_C;         //h
    display.setBrightness(brightness);
    if (B2cnt>0){
      if (digitalRead(B2_PIN)) B2cnt=0;
      if (B2cnt == 0) {
        brightness-=1; if (brightness==6) brightness=7;
        writeToEeprom(0, brightness);
      }
    }
    if (B3cnt>0){
      if (digitalRead(B3_PIN)) B3cnt=0;
      if (B3cnt == 0) {
        brightness+=1; if (brightness==12) brightness=11;
        writeToEeprom(0, brightness);
      }
    }
    
    if (B1cnt>0){ //--------------------------------
      if (digitalRead(B1_PIN)) B1cnt=0;
      if (B1cnt == 0){
        configState = CONFIG_ALRM_TYPE;
        blinkMinuteDigits = false;
        blinkHourDigits = false;
        blinkColumnDots = false;
      }
    }
    break;
    
    case CONFIG_ALRM_TYPE:///////////////////////////
//      A
//     ---
//  F |   | B
//     -G-
//  E |   | C
//     ---
//      D
    switch(alarmType){
    case BEEP:
      data[0] = SEG_F | SEG_E | SEG_D | SEG_C | SEG_G; //b
      data[1] = SEG_A | SEG_F | SEG_E | SEG_D | SEG_G; //E
      data[2] = SEG_A | SEG_F | SEG_E | SEG_D | SEG_G; //E
      data[3] = SEG_E | SEG_F | SEG_A | SEG_B | SEG_G; //p
      break;
      
    case LED1:
      data[0] = SEG_F | SEG_E | SEG_D;                 //L
      data[1] = SEG_A | SEG_F | SEG_E | SEG_D | SEG_G; //E
      data[2] = SEG_G | SEG_E | SEG_D | SEG_C | SEG_B; //d
      data[3] = SEG_B | SEG_C;                         //1
      break;
      
    case LED2:
      data[0] = SEG_F | SEG_E | SEG_D;                 //L
      data[1] = SEG_A | SEG_F | SEG_E | SEG_D | SEG_G; //E
      data[2] = SEG_G | SEG_E | SEG_D | SEG_C | SEG_B; //d
      data[3] = SEG_A | SEG_B | SEG_G | SEG_E | SEG_D; //2    
      break;
      
    case LED3:
      data[0] = SEG_F | SEG_E | SEG_D;                 //L
      data[1] = SEG_A | SEG_F | SEG_E | SEG_D | SEG_G; //E
      data[2] = SEG_G | SEG_E | SEG_D | SEG_C | SEG_B; //d
      data[3] = SEG_A | SEG_B | SEG_C | SEG_D | SEG_G; //3    
      break;
    }

    if (B2cnt>0){ //--------------------------------
      if (digitalRead(B2_PIN)) B2cnt=0;
      if (B2cnt == 0) {
        alarmType-=1; if (alarmType==BEEP-1) alarmType=BEEP;
        writeToEeprom(1, alarmType);
      }
    }
    if (B3cnt>0){ //--------------------------------
      if (digitalRead(B3_PIN)) B3cnt=0;
      if (B3cnt == 0) {
        alarmType+=1; if (alarmType==LED3+1) alarmType=LED3;
        writeToEeprom(1, alarmType);
      }
    }
    
    if (B1cnt>0){ //--------------------------------
      if (digitalRead(B1_PIN)) B1cnt=0;
      if (B1cnt == 0){
        configState = CONFIG_IDLE;
        blinkMinuteDigits = false;
        blinkHourDigits = false;
        blinkColumnDots = true;
      }
    }
    break;
  }
}


void loop()
{
  display.setBrightness(brightness);
  clear();
  
  while (1){
    
    delay(50);do_state_machine(); executeAlarm(); 
    delay(50);do_state_machine(); executeAlarm();
    delay(50);do_state_machine(); executeAlarm();
    delay(50);do_state_machine(); executeAlarm();
    delay(50);do_state_machine(); executeAlarm();
    delay(50);do_state_machine(); executeAlarm();
    delay(50);do_state_machine(); executeAlarm();
    delay(50);do_state_machine(); executeAlarm();
    delay(50);do_state_machine(); executeAlarm();
    delay(50);do_state_machine(); executeAlarm();
    
    blinkColumnOrDigits();
    
    if (!forceShutdownDisplayAndAlarmLed){
      display.setSegments(data);
      digitalWrite(PIN_ALRMLED, Clock.checkAlarmEnabled(1));
    } else {
      clear();
      digitalWrite(PIN_ALRMLED, LOW);
    }
    
  }
}
